#!/usr/bin/perl
### hud_status_monitor.pl
### Copyright 2008, Fonality Inc. ALL RIGHTS RESERVED.

### Connects to the HUD server and puts the status of connected HUD clients
### into the Asterisk DB via the Asterisk remote manager

### Data is stored in asterisk in the $db_tree family.
### The key is the extension, the status is either online, away, or the family/key doesn't exist for offline.
### e.g. /HUDStatus/8999 : online

### Matt Stofko <mstofko@fonality.com>

use POE qw(Component::IRC);
use IO::Socket;
use strict;

# Turn on for debugging output
my $DEBUG = 0;

# Make sure we're not already running (taken from desktop.pl)
(my $proc = $0) =~ s|^.*?([^/]+)$|$1|;
chomp(my $procs = `ps auwx|grep $proc|grep -cv grep`);
if ($procs > 1)
{
	die "$proc already running! Exiting...\n";
}

# Fork unless we have debugging turned on
if (!$DEBUG)
{
	&daemonize;
}

# determine our ami password
my $password;
my $username = 'local_mgr';
open(MGR, "</etc/asterisk/manager.conf") || die "Can't read /etc/asterisk/manager.conf: $!";
while (<MGR>)
{
	if (/^\[$username\]/)
	{
		while (<MGR>)
		{
			if (/^secret\s*=\s*(\S+)/)
			{
				$password = $1;
				last;
			}
		}
		last;
	}
}
close(MGR);
if (!$password)
{
	die "Can't find AMI password for local_mgr in /etc/asterisk/manager.conf!";
}

# File that contains the HUD server password
my $pwfile = '/usr/local/fonality/hud/auth.cfg';

# Channels on the HUD server that we need to connect to
my @channels = qw(#device_status #call_parking #call_incoming #transfer #call_action #device_state #upgrade);

# The Asterisk database family we will be writing to
my $db_tree = 'HUDStatus';

# Socket to connect to ARM, and storage for replies
my ($sock, %reply);

connect_to_arm();

# Set up our IRC stuff
my $irc = POE::Component::IRC->spawn;
POE::Session->create(
					inline_states => 
					{
						_start				=> \&mon_start,
						irc_001				=> \&on_connect,
						irc_public			=> \&on_public,
						irc_msg				=> \&on_msg,
						irc_disconnected	=> \&irc_disc,
						irc_socketerr		=> \&irc_sockerr
   					}
);

# Run it
$poe_kernel->run();
exit 0;

##### subs #####

sub irc_connect
{
	$irc->yield( register => "all" );

	# Make sure we have a pretty unique nick
    my $nick = 'statmon' . $$ % 1000;

	$irc->yield
	(
				connect =>
				{
					Nick     => $nick,
					Username => 'fmfm_mon',
					Ircname  => 'fmfm_mon',
					Server   => 'localhost',
					Port     => '6600',
					Password => get_password()
				}
    );

	return $nick;
}

# Connect to the server
sub mon_start
{
	my $nick = irc_connect();

	if ($DEBUG) { print(STDERR "Setting up connection. Nick is $nick.\n"); }
}

# The bot has successfully connected to a server.  Join a channel.
sub on_connect
{
	# Delete current entries
	%reply = send_to_arm("Action: Command\nCommand: database deltree $db_tree\n");
	if ($DEBUG) { print(STDERR "Deleting old entries.\n"); print(STDERR "$_ => $reply{$_}\n") foreach (keys %reply); }

	if ($DEBUG) { print(STDERR "Connected. Joining channels...\n"); }
	# Join all the channels
	foreach (@channels)
	{
		$irc->yield( join => $_);
		if ($DEBUG) { print(STDERR "Joined $_\n"); }
	}
}

# Handle disconnect events
sub irc_disc
{
	if ($DEBUG) { print(STDERR "Disconnected from server. Reconnecting...\n"); }

	# Call reconnect sub
	irc_connect();
}

# Handle socket errors which include disconnects and failed connects
sub irc_sockerr
{
	# Wait 5 seconds then try to reconnect
	sleep 5;
	irc_disc();
}

# Watch for public chat and process HUD status messages
sub on_public
{
	my ($kernel, $who, $where, $msg) = @_[KERNEL, ARG0, ARG1, ARG2];

	# Look for a status string
	if ($msg =~ m#^chat_member.*?((<s>\w+</s>).*?(<ext>\d+</ext>)|(<ext>\d+</ext>).*?(<s>\w+</s>))#)
	{
		if ($DEBUG) { print STDERR "Processing this line:\n\t$msg\n"; }

		# Grab the ext and status parts
		my $cmd = $1;

		# Parse out the extension
		$cmd =~ /<ext>(\d+)<\/ext>/;
		my $extension = $1;

		# And the status
		$cmd =~ /<s>(\w+)<\/s>/;
		my $status = $1;

		# If the new status isn't 'offline', set it in the DB
		if ($status ne 'offline')
		{
			if ($DEBUG) { print STDERR "Changing state on $extension to $status.\n"; }
			send_to_arm("Action: Command\nCommand: database put $db_tree $extension $status\n");
		}
		# Otherwise delete the entry from the DB (nonexistant key = offline)
		else
		{
			if ($DEBUG) { print STDERR "Changing state on $extension to $status. (Deleting out of the DB)\n"; }
			send_to_arm("Action: Command\nCommand: database del $db_tree $extension\n");
		}
    }
}

# Watch for private messages. The relay sends the current status of all online clients via private
# message when we connect and join all the channels
sub on_msg
{
	my ($kernel, $who, $where, $msg) = @_[KERNEL, ARG0, ARG1, ARG2];

	if ($msg =~ m#^chat_member.*?((<s>\w+</s>).*?(<ext>\d+</ext>)|(<ext>\d+</ext>).*?(<s>\w+</s>))#)
	{
		if ($DEBUG) { print STDERR "Processing this line:\n\t$msg\n"; }

		# Grab the ext and status parts
		my $cmd = $1;

		# Parse out the extension
		$cmd =~ /<ext>(\d+)<\/ext>/;
		my $extension = $1;

		# And the status
		$cmd =~ /<s>(\w+)<\/s>/;
		my $status = $1;

		if ($DEBUG) { print STDERR "Initial setting of state on $extension to $status.\n"; }
		# Set it in the DB
		send_to_arm("Action: Command\nCommand: database put $db_tree $extension $status\n");
    }
}

# Gets data from an asterisk manager response and returns it in a readable form
# (Taken from desktop.pl)
sub get_data
{
	my %data;

	while (<$sock>)
	{
		my @pair = split(/:\s*/, $_, 2);
		
		$pair[0] = lc($pair[0]);
		# if there's data
		if (@pair == 2)
		{
			$pair[1] =~ s/\r?\n$//;
			$data{$pair[0]} = $pair[1];
		}
		# end of data from server
		else
		{
			last;
		}
	}

	return %data;
}       

# Sends data to asterisk manager how it likes it
# (Taken from desktop.pl)
sub ssend
{
	my $data = shift;
	$data =~ s/\n/\r\n/gs;

	print $sock $data . "\r\n";
}

# Send a transaction to the asterisk remote manager and read its output
sub send_to_arm
{
	my $cmd = shift;

	# Generate some traffic so the block below can actually tell if our connection is down before we try to send something
	if ($cmd !~ /^Action: login|database del/)
	{
		ssend("\n\n");
		get_data();
	}

	# Check if we are connected, reconnect if not
	if (!$sock or !$sock->connected)
	{
		connect_to_arm();
	}

	# Send the data and return the reply
	ssend("$cmd\n");
	return get_data();
}

# Get the HUD server password for this PBXtra/trixbox
sub get_password
{
	open(F, $pwfile) or die "Cannot get HUD password from file: $!\n";
	my $password = <F>;
	close(F);
	chomp($password);

	return $password;
}

# Standard friendly perl forking
sub daemonize
{
	use POSIX 'setsid';
	chdir('/')               or die "Can't chdir to /: $!";
	open(STDIN, '/dev/null') or die "Can't read /dev/null: $!";
	open(STDOUT, '>/dev/null')
					   or die "Can't write to /dev/null: $!";
	defined(my $pid = fork) or die "Can't fork: $!";
	exit if $pid;
	setsid                  or die "Can't start a new session: $!";
	open(STDERR, '>&STDOUT') or die "Can't dup stdout: $!";
}

# Conntinually try to connect to the asterisk remote manager until successful
sub connect_to_arm
{
	while (!$sock or ($sock and !$sock->connected))
	{
		if ($DEBUG) { print STDERR "ARM connection down. Trying to reconnect.\n"; }

		# Set up a connection to Asterisk
		$sock = IO::Socket::INET->new
		(
				PeerAddr => 'localhost',
				PeerPort => 5038,
				Timeout => 10,
				Proto => 'tcp'
		);

		sleep(5) if !$sock or ($sock and !$sock->connected);
	}

	# Read and discard the ARM banner
	<$sock>;

	if ($DEBUG) { print STDERR "ARM connection successful.\n"; }

	# Authenticate
	%reply = send_to_arm("Action: login\nUsername: $username\nSecret: $password");
	if ($DEBUG) { print(STDERR "Logging in.\n"); print(STDERR "$_ => $reply{$_}\n") foreach (keys %reply); }
}
