#!/usr/bin/perl
#
# Copyright 2007, Fonality Inc. ALL RIGHTS RESERVED.
# Copyright (C) 2007  Fonality, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author: Adrian Otto <support@fonality.com>
# Description: Utility for setting the IP address locally


use strict;
use POSIX qw/setsid/;
use Getopt::Long;

# where most of the ip subs are:
require "/var/adm/bin/ip_funcs.pl";

close STDERR;

setsid();

my($ip,$sn,$gw,$nw,$bc,$d1,$d2,$dns,$mode,$dhcpd2);
GetOptions( 
	    'mode=s' => \$mode,
	    'ip=s' => \$ip,
	    'sn=s' => \$sn,
	    'gw=s' => \$gw,
	    'nw=s' => \$nw,
	    'bc=s' => \$bc,
	    'd1=s' => \$d1,
	    'd2=s' => \$d2,
	    'dns=s' => \$dns,
	    'dhcpd2=s' => \$dhcpd2,
	    );

my $settings = {
    ip => $ip,
    sn => $sn,
    gw => $gw,
    nw => $nw,
    bc => $bc,
    d1 => $d1,
    d2 => $d2,
    dns => $dns,
    dhcpd2 => $dhcpd2,
};

my($hostname) = `/bin/hostname`; chomp($hostname);

# save the old settings in case we want to revert
my ($old_mode, $old_settings) = get_current_config();

print STDERR $settings->{ip}.' '.uc($mode)."\n";

# try to run with the new settings
change_config(uc($mode), $settings);

sleep(10);

# can we do interweb stuff with these new settings?
my $result = on_internet($settings);

if ($result->[0] == 1)
{
    # ip settings were a success
    print 1;
    print STDERR localtime()."honky dory: ".$result->[1]."\n";
}
else
{
    print STDERR localtime()."failed: ".$result->[1]."\n";

    # bad ip settings, time to revert

    open my $fh, ">>/tmp/ip_restored" or die $!;
    print $fh localtime()."\n";
    print $fh "Error: $result->[1]\n";
    print $fh "Restored Settings: \n";
    
    for( qw/ip sn gw nw bc d1 d2/ )
    {
	print $fh "$_: $old_settings->{$_}\n";
    }
    close $fh;

    change_config($old_mode,$old_settings);
    print 0;
}

exit(0);



sub get_current_config
{
	my $mode = shift;
	my %setting = %{ $_[0] || {}};
	$setting{eth} ||= 'eth0';

	$setting{ip} = _get_ip_address($setting{eth})    unless exists $setting{ip};
	$setting{sn} = _get_netmask($setting{eth})       unless exists $setting{sn};
	$setting{gw} = _get_default_route() unless exists $setting{gw};
	       
	$setting{nw} = _get_network_addr($setting{ip})   unless exists $setting{nw};
	$setting{bc} = _get_broadcast_addr($setting{ip},$setting{eth}) unless exists $setting{bc};

	my ($ns) = _get_nameservers();
	$setting{d1} = $ns->[0] unless exists $setting{d1};
	$setting{d2} = $ns->[1] unless exists $setting{d2};
	if($setting{eth} eq 'eth1')
	{
	    $setting{dhcpd2} = _get_dhcpd2();
	}

	$mode = _get_bootproto() unless $mode;

	return($mode, \%setting);
}

##############################################################################
# change_config
##############################################################################
sub change_config
{
	my ($mode, $setting) = @_;

	if($mode eq 'STATIC')
	{
		system("killall -9 dhclient");
	}

	# Do the config stuff in one place so vtuns are started/stopped
	# correctly
	net_stop();
	if ($mode eq 'STATIC')
	{
		enable_static_ip($setting || {});
	}
	else
	{
		enable_dhcp_client($setting);
	}
	net_start();

	if( $mode eq 'STATIC' )
	{
			system( "route add default gw $setting->{gw}" );
	}
}

sub ping 
{
	my( $host ) = @_;

	my $ret = system( "/bin/ping -c 1 \Q$host\E > /dev/null 2>&1" );

	return 1 if $ret == 0;
}

sub arping 
{
	my( $host ) = @_;
	local $?;

	system("/sbin/arping -D -c 1 \Q$host\E > /dev/null 2>&1");
	return($?>>8);
}

sub on_internet
{
	my $settings = shift;
	open my $fh, "< /etc/sysconfig/vpn" or die $!;
	
	unless($settings->{gw})
	{
	    $settings->{gw} = _get_default_route();
	}

	my $host;
	while(<$fh>){ if( /GW1=(.+)/ ) { $host = $1; } }

	if( not arping($settings->{gw}) )
	{
	    return [0,"Failed to ping Gateway: $settings->{gw}"];
	}

	if( not ping( '4.2.2.6' ) )
	{
	    return [0, "Failed to ping to external DNS server (4.2.2.6)"];
	}
	

	if( not ping( $host ) )
	{
	    return [0,"Failed to ping internal VPN server ($host)"];
	}

	return [1];
}

close STDERR;
