use strict;

use Mail::Sender;
use Sys::Hostname;

my $DEBUG = 0;

my ($consecutive_ticks, $alreadyticked, $alarm_hash) = ({},{},{}); 

my %consecutive_ticks;
my %alreadyticked;
my %alarm_hash;
my $last_mtime;
my $sender;

my $hostname = get_host();

my $astcommands = [
    {
        '1.2'   => 'show channels',
        '1.6'   => 'core show channels',
    }, {
        '1.2'   => 'show channel ',
        '1.6'   => 'core show channel ',
    }, {
        '1.2'   => 'show dialplan ',
        '1.6'   => 'dialplan show ',
    }, {
        '1.2'   => 'show uptime',
        '1.6'   => 'core show uptime',
    }, {
        '1.2'   => 'show version',
        '1.6'   => 'core show version',
    }, {
        '1.2'   => 'show queues',
        '1.6'   => 'queue show',
    }, {
        '1.2'   => 'show queue ',
        '1.6'   => 'queue show ',
	}, {
		'1.2'	=> 'flush queue stats',
		'1.6'	=> 'queue flush stats',
    }, {
        '1.2'   => 'zap ',
        '1.6'   => 'dahdi ',
    }
];

# get our asterisk version
my $fullver = `/usr/sbin/asterisk -V`;
my ($version) = ($fullver =~ /(\d+\.\d+)/);

1;

sub get_host
{
	# look for hostname
	my $hostname = hostname;
	$hostname =~ s/pbxtra//;
	if($hostname !~ /^\d{3}/)
	{
		open (FILE, "</etc/asterisk/fonality/globals.conf");
		while(<FILE>)
		{
			($hostname) = $_ =~ /^ACCOUNT\s*=\s*(\d+)/;
		}
	}

	return $hostname;
}


sub get_conf
{
	my $alerts_file = (-e "/etc/asterisk/alerts2.conf" ? "/etc/asterisk/alerts2.conf" : "/etc/asterisk/alerts.conf");
	my $conf = shift;
	my ($mtime) = (stat($alerts_file))[9];
	if ($mtime > $last_mtime || !$conf)
	{
		$last_mtime = $mtime;
		$conf = {};

		open(FILE, "<$alerts_file");
		while (<FILE>)
		{
				if (m/^\s*(\w+)_(\w+)\s*=\s*(\S.*?)\s*$/)
				{
						$conf->{$1}->{$2} = $3;
				}
				elsif (/^\s*(\S+)\s*=\s*(\S.*?)\s*$/)
				{
						$conf->{$1} = $2;
				}
		}
		close(FILE);

		# look up e-mail address and create mail object for alerts
		open(FILE, "</etc/asterisk/voicemail.conf");
		while (<FILE>)
		{
			if (m/^\s*serveremail\s*=\s*(.*?)\s*$/)
			{
				$conf->{from} = $1;
				last;
			}
		}
		close(FILE);

		$sender = new Mail::Sender { smtp => 'localhost', from => $conf->{from} };

	}
	return $conf;
}


# send alarm
#
# args: to, type, percent
# checks last send, counts ticks, and sends e-mail


sub send_alarm
{
	my $to = shift;

	my $type      = shift;
	my $percent   = shift;
	my $conf_type = shift;
	my $conf      = shift;
	my $tick      = shift;
	$tick ||= 2;

	my $subject = $conf->{subject};
	my $message = $conf->{message};

	my $branding_software_name = $conf->{brandingname};

	my $time = time();
	my $lbs  = get_bup_status()->{'is_backup'};


	# is this a consecutive tick or a new line of ticks?
	# NOTE: sometimes it takes long enough to process that a second is skipped.
	# for the future this might want to be profiled and sped up a bit?
	if ($consecutive_ticks{$type}{lasttime} >= $time - $tick)
	{
		$consecutive_ticks{$type}{times}++;
		$consecutive_ticks{$type}{lasttime} = $time;
	}
	else
	{
		$consecutive_ticks{$type}{times} = 1;
		$consecutive_ticks{$type}{lasttime} = $time;
	}

	# is our last alarm greater than an hour ago and do we have 5 consecutive ticks?
	if ($consecutive_ticks{$type}{times} >= 5 && !$alreadyticked{$type})
	{
		# we tick this alarm to track how many real ticks (5+ consecutive ticks) per hour
		$alarm_hash->{$type}->{alarm_tick}++;

		$alreadyticked{$type} = 1;
		# only send an alarm every hour
		if ($time > $alarm_hash->{$type}->{last_alarm}+(60*60))
		{
			$subject =~ s/\[SERVER_ID\]/$hostname/g;	
			$subject =~ s/\[TYPE\]/$type/g;	
			$subject =~ s/\[USAGE\]/$percent/eg;	
			$subject =~ s/\[NUMALERTS\]/$alarm_hash->{$type}->{alarm_tick}/g;	
			$subject =~ s/\[BRANDING_SOFTWARE_NAME\]/$branding_software_name/g;	

			$message =~ s/\[SERVER_ID\]/$hostname/g;	
			$message =~ s/\[TYPE\]/$type/g;	
			$message =~ s/\[USAGE\]/$percent/eg;	
			$message =~ s/\[NUMALERTS\]/$alarm_hash->{$type}->{alarm_tick}/g;	
			$message =~ s/\[BRANDING_SOFTWARE_NAME\]/$branding_software_name/g;	

			if ($conf->{$conf_type}->{enable} && !$lbs && !$DEBUG)
			{
				foreach $to (split(/[;,]/,$to)) 
				{
					$sender->MailMsg({
						to => $to,
   						subject => $subject,
						msg =>   $message
					});
				}
			}

			$message =~ s/\n+/ /g;

			addlog($message);

			$alarm_hash->{$type}->{alarm_tick} = 0;	
			$alarm_hash->{$type}->{last_alarm} = $time;
		}
	}

	# we already ticked last time
	elsif ($consecutive_ticks{$type}{times} >= 5)
	{
		# this should remain one
		$alreadyticked{$type} = 1;
	}

	else
	{
		$alreadyticked{$type} = 0;
	}
}


sub round
{
        return sprintf("%.0f", $_[0]);
}


sub addlog
{
	open LOG, ">>/var/log/alerts.log";
	print LOG localtime() . "\n";
	print LOG $_[0] . "\n\n";
	close LOG;
}


# Putting this here will let it be accessible to server_stats_resources and server_stats_activity as well
sub get_bup_status
{
	my %status;
	$status{'is_backup'}  = -e '/etc/asterisk/primary' ? 1 : 0;
	$status{'has_backup'} = -e '/etc/asterisk/standby' ? 1 : -e '/etc/standby' ? 1 : 0;
	$status{'partofpair'} = $status{'is_backup'} || $status{'has_backup'} ? 1 : 0;

	return wantarray ? %status : \%status;
}


sub convert_cmd
{
    my $cmd = shift;
    return $cmd unless $version;

    # remove extra whitespace from command
    $cmd =~ s/\s+/ /g;

    # loop through command sets
    foreach my $cmdlist (@{$astcommands})
    {
        # go through each possible version
        foreach my $vers (keys %{$cmdlist})
        {
            # ignore our own
            next if $vers eq $version;

            # swap our command if we found one that matches
            if ($cmdlist->{$version} && $cmd =~ s/^$cmdlist->{$vers}/$cmdlist->{$version}/)
            {
                return $cmd;
            }
        }
    }

    return $cmd;
}

1;
