#!/usr/bin/perl
use strict;
use Net::Ping;

# Copyright 2006 Fonality, Inc. ALL RIGHTS RESERVED

system("/usr/bin/renice 5 $$ > /dev/null"); # Run at low priority (done from cron)

my %file = (
	pid     => '/var/run/sync.pid',
	oldpeer => '/etc/standby',
	peer    => '/etc/asterisk/standby',
	backup  => '/etc/asterisk/primary',
	tmpiax  => '/tmp/iax.conf',
	tmpsip  => '/tmp/sip.conf',
	iax     => '/etc/asterisk/iax.conf',
	sip     => '/etc/asterisk/sip.conf',
	globals => '/etc/asterisk/fonality/globals.conf',
	log     => '/var/log/sync_redundant.log',
);

clear_log();
log_action("\n\n----------\n\n" . localtime() . ": Starting new sync attempt.\n", "Creating pid file.");
create_pid_file($file{pid});

# If we're a backup server, put the iax and sip conf files into place correctly
if (-e $file{backup})
{
	log_action("This is a backup server, finding and de-registering voip accounts.");
	my %voip_accts = get_voip_info(globals => $file{globals});
	if (keys %voip_accts)
	{
		if (-e $file{tmpiax})
		{
			comment_voip(tmp_file => $file{tmpiax}, file => $file{iax}, voip => \%voip_accts);
		}
		if (-e $file{tmpsip})
		{
			comment_voip(tmp_file => $file{tmpsip}, file => $file{sip}, voip => \%voip_accts);
		}
	}
}

my $litmus = -e $file{peer} ? $file{peer} : -e $file{oldpeer} ? $file{oldpeer} : 0;
exit unless $litmus;
log_action("Peer file ($litmus) exists so we are continuing with sync.");

# Get the host from the peer file (whichever we happen to have)
open my $peer, '<', $litmus || die $!;
my ($host) = <$peer>;
close $peer;
chomp $host;

# We didn't have the 'b' in the entry, unlink the peer file we had and write out to the proper one incluing the 'b'.
if ($host =~ /(s\d{4,6})(b?)(.*)/ && $2 ne 'b')
{
	$host = "$1b$3";
	unlink $litmus;
	open my $write_peer, '>', $file{peer} || die $!;
	print $write_peer $host;
	close $write_peer;
}
log_action("From the peer file we are going to try to sync to '$host'");

# Use an icmp ping instead of a tcp ping, because otherwise a firewall might block the random port used for the tcp ping
# This would give a false negative. Potentially this should instead ping port 22 specifically?
my $p = Net::Ping->new('icmp');
my $status = $p->ping($host);
$p->close();
die "$host is not up. Not syncing.\n" unless $status;
log_action("Host is up, continuing with sync attempt.");

# list of files to exclude. Specials will still be synced, but to a different directory.
my @excludes = qw( standby primary );
my @specials = qw( iax.conf sip.conf );

my $ssh = "ssh -oStrictHostKeyChecking=no -oPasswordAuthentication=no";
my $rsync = "/usr/bin/rsync -e '$ssh' -aq --delete " . join ' ', map { " --exclude $_ " } @excludes;
my $rsync_special = $rsync . join ' ', map { " --exclude $_ " } @specials;
log_action("Using rsync as: $rsync");

my %dirs = (
	"/tftpboot"                        => "/",
	"/etc/postfix"                     => "/etc",
	"/etc/asterisk"                    => "/etc",
	"/etc/localtime"                   => "/etc",
	"/var/lib/asterisk/*db"            => "/var/lib/asterisk",
	"/etc/asterisk/sip.conf"           => "/tmp",
	"/etc/asterisk/iax.conf"           => "/tmp",
	"/var/lib/asterisk/sounds"         => "/var/lib/asterisk",
	"/var/lib/asterisk/mohmp3"         => "/var/lib/asterisk",
	"/var/spool/asterisk/monitor"      => "/var/spool/asterisk",
	"/var/adm/bin/pbxtra-fast.sh"      => "/var/adm/bin",
	"/var/spool/asterisk/voicemail"    => "/var/spool/asterisk",
	"/usr/local/ircd/etc/ircd.conf"    => "/usr/local/ircd",
	"/usr/local/fonality/hud/auth.cfg" => "/usr/local/fonality/hud",
	"/var/named/chroot/etc/named.conf" => "/var/named/chroot/etc",
);

my ($src, $dst);
while (($src, $dst) = each %dirs)
{
	my $cmd = grep({ $src =~ /\Q$_\E/ } @specials) ? $rsync : $rsync_special;
	$cmd .= " $src $host:$dst";
	log_action("Syncing $src" . (' 'x(30 - length($src))) . "-> $dst");
	system($cmd);
}

unlink $file{pid};
log_action("Restarting asterisk on host $host.");
system("/usr/bin/ssh $host '/usr/sbin/asterisk -r -x \"restart when convenient\"'");

sub create_pid_file {
	my($pidfile) = shift(@_);
	if(-f $pidfile) {
		open(PID, $pidfile) || die "Unable to open $pidfile: $!";
		my(@pid_contents) = <PID>;
		close(PID);
		chomp(@pid_contents);
		if(kill(0,$pid_contents[0])) { # Another sync is still running
			system("/usr/bin/logger \"$0 found another instance pid=$pid_contents[0] running and killed it\"");
			kill(9,$pid_contents[0]); # Stop the other sync
			sleep(3); # Give subprocesses a moment to quit
		}
	}
	open(PID, ">$pidfile") || die "Unable to open $pidfile: $!";
	print PID "$$\n";
	close(PID);
}

sub get_voip_info
{
	my %arg = ref($_[0]) eq 'HASH' ? %{+shift} : @_;
	my %voip;

	open my $globals, '<', $arg{'globals'} or return;
	while (my $line = <$globals>)
	{
		if ($line =~ /^VOIP(\d)_([^\s=]+)\s*=\s*([^;\s]+)/i)
		{
			my ($account, $meta, $info) = map { lc } ($1, $2, $3);
			if (not exists $voip{$account})
			{
				$voip{$account} = {};
			}

			$voip{$account}{$meta} = $info;
		}
	}

	return wantarray ? %voip : \%voip;
}

sub comment_voip
{
	my %arg = ref($_[0]) eq 'HASH' ? %{+shift} : @_;

	open my $tmp_file, '<', $arg{'tmp_file'} or return;
	open my $file, '>', $arg{'file'} or do { close $tmp_file; return };

	while (my $line = <$tmp_file>)
	{
		if ($line =~ /register/)
		{
			$line = ';' . $line;
		}

		print $file $line;
	}

	close $file;
	close $tmp_file;
}

sub clear_log
{
	if (-e $file{'log'})
	{
		unlink $file{'log'};
	}
}

sub log_action
{
	my $text = join '', @_;
	open my $log, '>>', $file{'log'} or return;
	print $log "$text\n";
	close $log;
}
