#!/usr/bin/perl
use strict;

# Copyright 2008 Fonality, Inc. ALL RIGHTS RESERVED

use Sys::Hostname  qw(hostname);
use LWP::UserAgent ();
use Digest::MD5    ();
use List::Util   qw(minstr maxstr);

my $ignore_sleep = shift() ? 1 : 0;

# Flags to pass to rpm when verifying a package
my $rpm_verify_flags = '-V --nomtime --nogroup';

if (!$ignore_sleep)
{
	# So we don't tag the server(s) all at once
	sleep int rand(60*60*4);
}

my ($user, $pass) = get_auth();
my $sid           = get_sid();
my $rpm_list      = get_rpm_list();
my $info          = get_rpm_info($rpm_list);

# If we have no packages out of sync, exit
if (ref($info) ne 'HASH' || keys(%$info) < 1)
{
	exit;
}

post_url($sid, $user, $pass, $info);

sub post_url
{
	my ($server_id, $user, $passwd, $pkgs) = @_;
	my $url = $server_id < 100_000 ? 'http://cp.fonality.com/' : 'http://cp.trixbox.com/';
	$url   .= "update_check.cgi?server_id=$server_id&user=$user&hash=$passwd";

	foreach my $pkg (keys %$pkgs)
	{
		$url .= "\&$pkg=$pkgs->{$pkg}{version}";
		if (exists($pkgs->{$pkg}{reason}) && $pkgs->{$pkg}{reason} eq 'invalid')
		{
			$url .= ',invalid';
		}
	}

	my $lwp_request = LWP::UserAgent->new();
	$lwp_request->get($url);
}

# TODO: This needs to be changed to by dynamic in the future!
sub get_rpm_list
{
	# Single quote the backtick operator so it doesn't interpolate before passing to shell
	my $ret = `rpm -qa --queryformat="%{N}#%{V}#%{RELEASE}\n" | grep "HUD\\|FON"`;
	return wantarray ? split(/\n/, $ret) : $ret;
}

# construct the info; ignore "is not installed" warnings since they may not have gotten the upgrade to install a particular package
sub get_rpm_info
{
	my $raw_info = shift;

	my %info;
	foreach my $package (split /\n/, $raw_info)
	{
		next unless $package =~ /#/;
		chomp($package);
		my ($version, $release);

		($package, $version, $release) = split /#/, $package;

		if (exists $info{$package})
		{
			my $min = minstr($info{$package}{version}, "$version-$release");
			system("/bin/rpm --erase --noscripts --notriggers $package-$min");
			$info{$package}{version} = maxstr($info{$package}{version}, "$version-$release");
		}
		else
		{
			$info{$package} = { version => "$version-$release" };
		}

		my $oops = `rpm $rpm_verify_flags $package`;
		chomp($oops);

		# If there's a response that means there's a difference. If the response is "is not installed", ignore it.
		if ($oops && $oops !~ /is not installed/)
		{
			$info{$package}{reason} = 'invalid';
		}
	}

	return wantarray ? %info : \%info;
}

sub get_sid
{
	my ($server_id) = hostname() =~ /\A \D* (\d+) \z/xms;
	return $server_id;
}

sub get_auth
{
	my ($user, $pass) = ('', '');

	open my $globals, '<', '/etc/asterisk/fonality/globals.conf' || return($user, $pass);
	while (my $line = <$globals>)
	{
		if ($line =~ /^IAXINFO/)
		{
			if ($line =~ /IAXINFO\d?=(s\d+):([^;\s]+)/)
			{
				($user, $pass) = ($1, Digest::MD5::md5_hex($2));
				$user =~ s/[xi]$//;
			}
			last;
		}
	}

	close $globals;
	return($user, $pass);
}
